<?php
defined('ABSPATH') || exit;

/**
 * This action hook registers our PHP class as a WooCommerce payment gateway
 */
add_filter('woocommerce_payment_gateways', 'xprespay_add_gateway_class');
function xprespay_add_gateway_class($gateways)
{
    $gateways[] = 'WC_XpresPay_Gateway';
    return $gateways;
}

/**
 * The class itself, note that it is inside plugins_loaded action hook
 */
add_action('plugins_loaded', 'xprespay_init_gateway_class');
function xprespay_init_gateway_class()
{

    class WC_XpresPay_Gateway extends WC_Payment_Gateway
    {

        /**
         * Class constructor
         */
        public function __construct()
        {
            $this->id = 'xprespay'; // payment gateway plugin ID
            $this->icon = plugins_url('img/xprespay-payment-logo.svg', __FILE__);
            $this->has_fields = false; // in case a custom credit card form is needed
            $this->method_title = 'XpresPay';
            $this->method_description = 'Modtag betaling med XpresPay.'; // will be displayed on the options page

            // Method with all the options fields
            $this->init_form_fields();

            // Load settings
            $this->init_settings();
            $this->title = $this->get_option('title');
            $this->description = $this->get_option('description');
            $this->enabled = $this->get_option('enabled');
            $this->testmode = false;
            $this->private_key = '';
            $this->publishable_key = '';

            // Define user set variables
            if (WC()->cart && !is_admin() && $this->get_option('enabled') == 'yes') {
                if (get_woocommerce_currency() != 'DKK') {
                    $this->enabled = 'no';
                } else {
                    $order_total = ceil($this::get_order_total());
                    $loanInfo = SparxpresUtils::get_loan_information(SparxpresUtils::get_link_id(), $order_total);
                    if (isset($loanInfo)) {
                        $isXpresPayEnabled = SparxpresUtils::is_xprespay_enabled($loanInfo, $order_total);
                        if ($isXpresPayEnabled) {
                            $this->enabled = 'yes';
                        } else {
                            $this->enabled = 'no';
                        }
                    } else {
                        $this->enabled = 'no';
                    }
                }
            }

            // This action hook saves the settings (on parent)
            add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));
            // Action hook for the thank you page
            add_action('woocommerce_thankyou_' . $this->id, array($this, 'process_thank_you_page'));
        }

        /**
         * Plugin options
         */
        function init_form_fields()
        {
            $this->form_fields = array(
                'enabled' => array(
                    'title' => __('Enable/Disable', 'sparxpres-websale-plugin'),
                    'label' => __('Enable XpresPay', 'sparxpres-websale-plugin'),
                    'type' => 'checkbox',
                    'description' => '',
                    'default' => 'yes'
                ),
                'title' => array(
                    'title' => __('Title', 'sparxpres-websale-plugin'),
                    'type' => 'text',
                    'description' => __('This controls the title which the user sees during checkout.', 'sparxpres-websale-plugin'),
                    'default' => 'XpresPay',
                    'desc_tip' => true,
                ),
                'description' => array(
                    'title' => __('Description', 'sparxpres-websale-plugin'),
                    'type' => 'textarea',
                    'description' => __('Payment method description that the customer will see on checkout.', 'sparxpres-websale-plugin'),
                    'default' => 'Betal med din XpresPay kredit.' . PHP_EOL . 'Følg anvisningerne på næste side.',
                    'desc_tip' => true,
                )
            );
        }

        /**
         * We're processing the payments here
         * @param $order_id
         * @return array
         */
        function process_payment($order_id)
        {
            $order = wc_get_order($order_id);

            $linkId = SparxpresUtils::get_link_id();
            $amountCents = ceil($order->get_total() * 100);
            $return_url = $this->get_return_url($order);
            $cancel_url = $order->get_cancel_order_url();

            $order->add_order_note('Afventer XpresPay betaling.');

            return array(
                'result' => 'success',
                'redirect' => 'https://sparxpres.dk/app/xprespay/betal/'
                    . '?linkId=' . $linkId
                    . '&transactionId=' . $order_id
                    . '&amountCents=' . $amountCents
                    . '&returnurl=' . urlencode($return_url)
                    . '&cancelurl=' . urlencode($cancel_url)
            );
        }

        /**
         * Thank you page
         * @param $order_id
         */
        function process_thank_you_page($order_id)
        {
            if (empty($order_id)) {
                return;
            }

            // Get an instance of the WC_Order object
            $order = wc_get_order($order_id);
            if (!empty($order)) {
                if ($order->get_status() === 'pending') {
                    $order->update_status('on-hold', 'Anmodningen er sendt til XpresPay.');  // Awaiting payment – stock is reduced
                } else {
                    $order->add_order_note('Anmodningen er sendt til XpresPay.');
                }
            }
        }
    }
}
